defmodule Nulla.ActorsTest do
  use Nulla.DataCase

  alias Nulla.KeyGen
  alias Nulla.Actors

  describe "actors" do
    alias Nulla.Actors.Actor

    import Nulla.ActorsFixtures

    @invalid_attrs %{
      name: nil,
      tag: nil,
      type: nil,
      image: nil,
      url: nil,
      acct: nil,
      ap_id: nil,
      following: nil,
      followers: nil,
      inbox: nil,
      outbox: nil,
      featured: nil,
      featuredTags: nil,
      preferredUsername: nil,
      summary: nil,
      manuallyApprovesFollowers: nil,
      discoverable: nil,
      indexable: nil,
      published: nil,
      memorial: nil,
      publicKey: nil,
      privateKeyPem: nil,
      attachment: nil,
      endpoints: nil,
      icon: nil,
      vcard_bday: nil,
      vcard_Address: nil
    }

    test "list_actors/0 returns all actors" do
      actor = actor_fixture()
      assert Actors.list_actors() == [actor]
    end

    test "get_actor!/1 returns the actor with given id" do
      actor = actor_fixture()
      assert Actors.get_actor!(actor.id) == actor
    end

    test "create_actor/1 with valid data creates a actor" do
      username = "test#{System.unique_integer()}"
      {publicKeyPem, privateKeyPem} = KeyGen.gen()

      valid_attrs = %{
        name: "some name",
        tag: [],
        type: "some type",
        image: %{},
        url: "some url",
        acct: "#{username}@localhost",
        ap_id: "http://localhost/users/#{username}",
        following: "some following",
        followers: "some followers",
        inbox: "some inbox",
        outbox: "some outbox",
        featured: "some featured",
        featuredTags: "some featuredTags",
        preferredUsername: username,
        summary: "some summary",
        manuallyApprovesFollowers: true,
        discoverable: true,
        indexable: true,
        published: ~U[2025-06-30 13:31:00Z],
        memorial: true,
        publicKey: %{
          "id" => "http://localhost/users/#{username}#main-key",
          "owner" => "http://localhost/users/#{username}",
          "publicKeyPem" => publicKeyPem
        },
        privateKeyPem: privateKeyPem,
        attachment: [],
        endpoints: %{},
        icon: %{},
        vcard_bday: ~D[2025-06-30],
        vcard_Address: "some vcard_Address"
      }

      assert {:ok, %Actor{} = actor} = Actors.create_actor(valid_attrs)
      assert actor.name == "some name"
      assert actor.tag == []
      assert actor.type == "some type"
      assert actor.image == %{}
      assert actor.url == "some url"
      assert actor.acct == "#{username}@localhost"
      assert actor.ap_id == "http://localhost/users/#{username}"
      assert actor.following == "some following"
      assert actor.followers == "some followers"
      assert actor.inbox == "some inbox"
      assert actor.outbox == "some outbox"
      assert actor.featured == "some featured"
      assert actor.featuredTags == "some featuredTags"
      assert actor.preferredUsername == username
      assert actor.summary == "some summary"
      assert actor.manuallyApprovesFollowers == true
      assert actor.discoverable == true
      assert actor.indexable == true
      assert actor.published == ~U[2025-06-30 13:31:00Z]
      assert actor.memorial == true
      assert actor.publicKey["publicKeyPem"] == publicKeyPem
      assert actor.privateKeyPem == privateKeyPem
      assert actor.attachment == []
      assert actor.endpoints == %{}
      assert actor.icon == %{}
      assert actor.vcard_bday == ~D[2025-06-30]
      assert actor.vcard_Address == "some vcard_Address"
    end

    test "create_actor/1 with invalid data returns error changeset" do
      assert {:error, %Ecto.Changeset{}} = Actors.create_actor(@invalid_attrs)
    end

    test "update_actor/2 with valid data updates the actor" do
      actor = actor_fixture()
      username = "test#{System.unique_integer()}"
      {publicKeyPem, privateKeyPem} = KeyGen.gen()

      update_attrs = %{
        name: "some updated name",
        tag: [],
        type: "some updated type",
        image: %{},
        url: "some updated url",
        acct: "#{username}@localhost",
        ap_id: "http://localhost/users/#{username}",
        following: "some updated following",
        followers: "some updated followers",
        inbox: "some updated inbox",
        outbox: "some updated outbox",
        featured: "some updated featured",
        featuredTags: "some updated featuredTags",
        preferredUsername: username,
        summary: "some updated summary",
        manuallyApprovesFollowers: false,
        discoverable: false,
        indexable: false,
        published: ~U[2025-07-01 13:31:00Z],
        memorial: false,
        publicKey: %{
          "id" => "http://localhost/users/#{username}#main-key",
          "owner" => "http://localhost/users/#{username}",
          "publicKeyPem" => publicKeyPem
        },
        privateKeyPem: privateKeyPem,
        attachment: [],
        endpoints: %{},
        icon: %{},
        vcard_bday: ~D[2025-07-01],
        vcard_Address: "some updated vcard_Address"
      }

      assert {:ok, %Actor{} = actor} = Actors.update_actor(actor, update_attrs)
      assert actor.name == "some updated name"
      assert actor.tag == []
      assert actor.type == "some updated type"
      assert actor.image == %{}
      assert actor.url == "some updated url"
      assert actor.acct == "#{username}@localhost"
      assert actor.ap_id == "http://localhost/users/#{username}"
      assert actor.following == "some updated following"
      assert actor.followers == "some updated followers"
      assert actor.inbox == "some updated inbox"
      assert actor.outbox == "some updated outbox"
      assert actor.featured == "some updated featured"
      assert actor.featuredTags == "some updated featuredTags"
      assert actor.preferredUsername == username
      assert actor.summary == "some updated summary"
      assert actor.manuallyApprovesFollowers == false
      assert actor.discoverable == false
      assert actor.indexable == false
      assert actor.published == ~U[2025-07-01 13:31:00Z]
      assert actor.memorial == false
      assert actor.publicKey["publicKeyPem"] == publicKeyPem
      assert actor.privateKeyPem == privateKeyPem
      assert actor.attachment == []
      assert actor.endpoints == %{}
      assert actor.icon == %{}
      assert actor.vcard_bday == ~D[2025-07-01]
      assert actor.vcard_Address == "some updated vcard_Address"
    end

    test "update_actor/2 with invalid data returns error changeset" do
      actor = actor_fixture()
      assert {:error, %Ecto.Changeset{}} = Actors.update_actor(actor, @invalid_attrs)
      assert actor == Actors.get_actor!(actor.id)
    end

    test "delete_actor/1 deletes the actor" do
      actor = actor_fixture()
      assert {:ok, %Actor{}} = Actors.delete_actor(actor)
      assert_raise Ecto.NoResultsError, fn -> Actors.get_actor!(actor.id) end
    end

    test "change_actor/1 returns a actor changeset" do
      actor = actor_fixture()
      assert %Ecto.Changeset{} = Actors.change_actor(actor)
    end
  end
end
