defmodule Nulla.NotesTest do
  use Nulla.DataCase

  alias Nulla.Notes

  describe "notes" do
    alias Nulla.Notes.Note

    import Nulla.ActorsFixtures
    import Nulla.NotesFixtures

    @invalid_attrs %{
      sensitive: nil,
      cc: nil,
      to: nil,
      url: nil,
      language: nil,
      inReplyTo: nil,
      published: nil,
      visibility: nil,
      content: nil,
      actor_id: nil
    }

    test "list_notes/0 returns all notes" do
      note = note_fixture()
      assert Notes.list_notes() == [note]
    end

    test "get_note!/1 returns the note with given id" do
      note = note_fixture()
      assert Notes.get_note!(note.id) == note
    end

    test "create_note/1 with valid data creates a note" do
      actor = actor_fixture()

      valid_attrs = %{
        sensitive: true,
        cc: ["option1", "option2"],
        to: ["option1", "option2"],
        url: "some url",
        language: "some language",
        inReplyTo: "some inReplyTo",
        published: ~U[2025-07-01 09:17:00Z],
        visibility: "some visibility",
        content: "some content",
        actor_id: actor.id
      }

      assert {:ok, %Note{} = note} = Notes.create_note(valid_attrs)
      assert note.sensitive == true
      assert note.cc == ["option1", "option2"]
      assert note.to == ["option1", "option2"]
      assert note.url == "some url"
      assert note.language == "some language"
      assert note.inReplyTo == "some inReplyTo"
      assert note.published == ~U[2025-07-01 09:17:00Z]
      assert note.visibility == "some visibility"
      assert note.content == "some content"
      assert note.actor_id == actor.id
    end

    test "create_note/1 with invalid data returns error changeset" do
      assert {:error, %Ecto.Changeset{}} = Notes.create_note(@invalid_attrs)
    end

    test "update_note/2 with valid data updates the note" do
      actor = actor_fixture()
      note = note_fixture()

      update_attrs = %{
        sensitive: false,
        cc: ["option1"],
        to: ["option1"],
        url: "some updated url",
        language: "some updated language",
        inReplyTo: "some updated inReplyTo",
        published: ~U[2025-07-02 09:17:00Z],
        visibility: "some updated visibility",
        content: "some updated content",
        actor_id: actor.id
      }

      assert {:ok, %Note{} = note} = Notes.update_note(note, update_attrs)
      assert note.sensitive == false
      assert note.cc == ["option1"]
      assert note.to == ["option1"]
      assert note.url == "some updated url"
      assert note.language == "some updated language"
      assert note.inReplyTo == "some updated inReplyTo"
      assert note.published == ~U[2025-07-02 09:17:00Z]
      assert note.visibility == "some updated visibility"
      assert note.content == "some updated content"
      assert note.actor_id == actor.id
    end

    test "update_note/2 with invalid data returns error changeset" do
      note = note_fixture()
      assert {:error, %Ecto.Changeset{}} = Notes.update_note(note, @invalid_attrs)
      assert note == Notes.get_note!(note.id)
    end

    test "delete_note/1 deletes the note" do
      note = note_fixture()
      assert {:ok, %Note{}} = Notes.delete_note(note)
      assert_raise Ecto.NoResultsError, fn -> Notes.get_note!(note.id) end
    end

    test "change_note/1 returns a note changeset" do
      note = note_fixture()
      assert %Ecto.Changeset{} = Notes.change_note(note)
    end
  end
end
