defmodule Nulla.RelationsTest do
  use Nulla.DataCase

  alias Nulla.Relations

  describe "relations" do
    alias Nulla.Relations.Relation

    import Nulla.ActorsFixtures
    import Nulla.RelationsFixtures

    @invalid_attrs %{
      following: nil,
      followed_by: nil,
      showing_replies: nil,
      showings_reblogs: nil,
      notifying: nil,
      muting: nil,
      muting_notifications: nil,
      blocking: nil,
      blocked_by: nil,
      domain_blocking: nil,
      requested: nil,
      note: nil,
      local_actor_id: nil,
      remote_actor_id: nil
    }

    test "list_relations/0 returns all relations" do
      relation = relation_fixture()
      assert Relations.list_relations() == [relation]
    end

    test "get_relation!/1 returns the relation with given id" do
      relation = relation_fixture()
      assert Relations.get_relation!(relation.id) == relation
    end

    test "create_relation/1 with valid data creates a relation" do
      local_actor = actor_fixture()
      remote_actor = actor_fixture()

      valid_attrs = %{
        following: true,
        followed_by: true,
        showing_replies: true,
        showings_reblogs: true,
        notifying: true,
        muting: true,
        muting_notifications: true,
        blocking: true,
        blocked_by: true,
        domain_blocking: true,
        requested: true,
        note: "some note",
        local_actor_id: local_actor.id,
        remote_actor_id: remote_actor.id
      }

      assert {:ok, %Relation{} = relation} = Relations.create_relation(valid_attrs)
      assert relation.following == true
      assert relation.followed_by == true
      assert relation.showing_replies == true
      assert relation.showings_reblogs == true
      assert relation.notifying == true
      assert relation.muting == true
      assert relation.muting_notifications == true
      assert relation.blocking == true
      assert relation.blocked_by == true
      assert relation.domain_blocking == true
      assert relation.requested == true
      assert relation.note == "some note"
      assert relation.local_actor_id == local_actor.id
      assert relation.remote_actor_id == remote_actor.id
    end

    test "create_relation/1 with invalid data returns error changeset" do
      assert {:error, %Ecto.Changeset{}} = Relations.create_relation(@invalid_attrs)
    end

    test "update_relation/2 with valid data updates the relation" do
      local_actor = actor_fixture()
      remote_actor = actor_fixture()
      relation = relation_fixture()

      update_attrs = %{
        following: false,
        followed_by: false,
        showing_replies: false,
        showings_reblogs: false,
        notifying: false,
        muting: false,
        muting_notifications: false,
        blocking: false,
        blocked_by: false,
        domain_blocking: false,
        requested: false,
        note: "some updated note",
        local_actor_id: local_actor.id,
        remote_actor_id: remote_actor.id
      }

      assert {:ok, %Relation{} = relation} = Relations.update_relation(relation, update_attrs)
      assert relation.following == false
      assert relation.followed_by == false
      assert relation.showing_replies == false
      assert relation.showings_reblogs == false
      assert relation.notifying == false
      assert relation.muting == false
      assert relation.muting_notifications == false
      assert relation.blocking == false
      assert relation.blocked_by == false
      assert relation.domain_blocking == false
      assert relation.requested == false
      assert relation.note == "some updated note"
      assert relation.local_actor_id == local_actor.id
      assert relation.remote_actor_id == remote_actor.id
    end

    test "update_relation/2 with invalid data returns error changeset" do
      relation = relation_fixture()
      assert {:error, %Ecto.Changeset{}} = Relations.update_relation(relation, @invalid_attrs)
      assert relation == Relations.get_relation!(relation.id)
    end

    test "delete_relation/1 deletes the relation" do
      relation = relation_fixture()
      assert {:ok, %Relation{}} = Relations.delete_relation(relation)
      assert_raise Ecto.NoResultsError, fn -> Relations.get_relation!(relation.id) end
    end

    test "change_relation/1 returns a relation changeset" do
      relation = relation_fixture()
      assert %Ecto.Changeset{} = Relations.change_relation(relation)
    end
  end
end
